// File OGLCtrl.cpp

#include "stdafx.h"
#include "OGLCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Include the libraries to link
#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")  // Perhaps not needed

//*****************************************************************************
// OpenGL reusable window classes to encapsulate windows with OpenGL graphs
// By G.Bavestrelli                           Copyright Techint S.p.A. 1998
//*****************************************************************************

IMPLEMENT_DYNAMIC(COpenGLCtrl,CWnd); 
IMPLEMENT_DYNAMIC(COpenGLCtrls,CWnd); 

//=============================================================================
// Construction, destruction and message maps
//=============================================================================

COpenGLCtrl::COpenGLCtrl()
            :m_ClipBox(0,0,0,0),
             m_hRC(NULL),m_hDC(NULL)
{
    static BOOL Registered=FALSE;
    if (!Registered)
         Registered=RegisterOpenGLWindowClass();
}

COpenGLCtrl::~COpenGLCtrl()
{
}

BEGIN_MESSAGE_MAP(COpenGLCtrl, CWnd)
    //{{AFX_MSG_MAP(COpenGLCtrl)
    ON_WM_DESTROY()
    ON_WM_SIZE()
    ON_WM_PAINT()
    ON_WM_ERASEBKGND()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

//=============================================================================
// Message handlers and other overridden MFC functions
//=============================================================================

BOOL COpenGLCtrl::PreCreateWindow(CREATESTRUCT& cs) 
{
    // Need these styles to use OpenGL
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
    return CWnd::PreCreateWindow(cs);
}

void COpenGLCtrl::PreSubclassWindow() 
{
   ASSERT(m_hWnd);
   // Window class must not have style CS_PARENTDC
   ASSERT((GetClassLong(m_hWnd,GCL_STYLE)&CS_PARENTDC)==0); 
   // Must add these styles to use OpenGL
   ModifyStyle(0,WS_CLIPSIBLINGS|WS_CLIPCHILDREN);

   InitializeOpenGLWindow();

   CRect Rect;
   GetClientRect(&Rect);

   // If we are subclassing an existing window 
   // we might have missed the WM_SIZE message
   if (!Rect.IsRectEmpty())
      ResizeOpenGLWindow(Rect.Width(),Rect.Height());

   CWnd::PreSubclassWindow();
}

void COpenGLCtrl::OnSize(UINT nType, int cx, int cy) 
{
    CWnd::OnSize(nType, cx, cy);
    
    if (cx>0 && cy>0)
       ResizeOpenGLWindow(cx,cy);
}

void COpenGLCtrl::OnPaint() 
{
    CPaintDC DC(this);

    if (DC.GetClipBox(&m_ClipBox)!=NULLREGION)
       RedrawOpenGLWindow();

    m_ClipBox.SetRectEmpty();
}   

BOOL COpenGLCtrl::OnEraseBkgnd(CDC* pDC) 
{
    return TRUE; // Don't clear background, OpenGL will
}

void COpenGLCtrl::OnDestroy() 
{
    UninitializeOpenGLWindow();
    CWnd::OnDestroy();
}

//=============================================================================
// Set Pixel Format and Register Window Class
//=============================================================================

void COpenGLCtrl::SetupPixelFormat(HDC hDC)
{
    PIXELFORMATDESCRIPTOR pfd = 
    {
        sizeof(PIXELFORMATDESCRIPTOR),// size of this pfd
        1,                      // version number
        PFD_DRAW_TO_WINDOW      // support window
        | PFD_SUPPORT_OPENGL    // support OpenGL
        | PFD_DOUBLEBUFFER      // double buffered 
        , PFD_TYPE_RGBA,        // RGBA type
        24,                     // 24-bit color depth
        0, 0, 0, 0, 0, 0,       // color bits ignored
        0,                      // no alpha buffer
        0,                      // shift bit ignored
        0,                      // no accumulation buffer
        0, 0, 0, 0,             // accum bits ignored
        32,                     // 32-bit z-buffer
        0,                      // no stencil buffer
        0,                      // no auxiliary buffer
        PFD_MAIN_PLANE,         // main layer
        0,                      // reserved
        0, 0, 0                 // layer masks ignored
    };

    // Give user a chance to change the pfd structure
    CustomizePixelFormat(&pfd);

    int  pixelformat=0;
    VERIFY(pixelformat=ChoosePixelFormat(hDC, &pfd));
    VERIFY(SetPixelFormat(hDC,pixelformat, &pfd));
}

BOOL COpenGLCtrl::RegisterOpenGLWindowClass()
{
    WNDCLASS wcls;

    // If already registered, return
    if (::GetClassInfo(AfxGetInstanceHandle(),"MyOpenGL", &wcls))
         return TRUE;

    wcls.style=CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
    wcls.lpfnWndProc=(WNDPROC)::DefWindowProc;
    wcls.cbClsExtra=0;
    wcls.cbWndExtra=0;
    wcls.hInstance=AfxGetInstanceHandle();
    wcls.hIcon=::LoadIcon(NULL,IDI_APPLICATION);
    wcls.hCursor=::LoadCursor(NULL,IDC_ARROW);
    wcls.hbrBackground=(HBRUSH)(COLOR_WINDOW+1);
    wcls.lpszMenuName=NULL;
    wcls.lpszClassName="MyOpenGL";

    return AfxRegisterClass(&wcls);
}

//=============================================================================
// Initialization and Cleanup of OpenGL
//=============================================================================

void COpenGLCtrl::InitializeOpenGLWindow()
{
    // Must initialize only once
    ASSERT(!m_hRC); 
    ASSERT(!m_hDC);
    ASSERT(m_hWnd && ::IsWindow(m_hWnd));

    // Get the DC and set its pixel format
    VERIFY(m_hDC=::GetDC(m_hWnd));
    SetupPixelFormat(m_hDC);

    // Create the Rendering Context and make it current
    VERIFY(m_hRC=wglCreateContext(m_hDC));
    VERIFY(wglMakeCurrent(m_hDC,m_hRC));

    // Let user do his one-time initialization
    OnCreateRC();
    ASSERT(!GetOpenGLError());
}

void COpenGLCtrl::UninitializeOpenGLWindow()
{
    ASSERT(m_hRC);
    ASSERT(m_hDC);

    // Delete the RC
    if (m_hRC)
    {   
        OnDestroyRC(); // Let derived classes clean up
        VERIFY(wglMakeCurrent(NULL,NULL));
        VERIFY(wglDeleteContext(m_hRC));
        m_hRC=NULL;
    }

    // Release the DC
    if (m_hDC)
    {
        ASSERT(m_hWnd);
        VERIFY(::ReleaseDC(m_hWnd,m_hDC));
        m_hDC=NULL;
    }
}

//=============================================================================
// Members that implement the protocol handling WM_SIZE and WM_PAINT
//=============================================================================

// Handle resizing of OpenGL window
void COpenGLCtrl::ResizeOpenGLWindow(int cx, int cy) 
{
    ASSERT(cx>0);
    ASSERT(cy>0);
    ASSERT(m_hDC);
    ASSERT(!GetOpenGLError());
    ASSERT(wglGetCurrentContext());
    ASSERT(wglGetCurrentDC()==m_hDC);

    // Give user a chance to set up his viewport transformation
    OnViewport(cx, cy);
    ASSERT(!GetOpenGLError());

    // Select projection matrix and initialize it
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // Get the aspect ratio
    GLdouble AspectRatio=(GLdouble)cx/(GLdouble)cy;
    ASSERT(AspectRatio>0);
    // Give user a chance to set his perspective transformation
    OnProjection(AspectRatio);
    ASSERT(!GetOpenGLError());

    // Select ModelView matrix and let user set viewing transform
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 
    // Give user a chance to set his viewing transformation
    OnViewing();
    ASSERT(!GetOpenGLError());
}

// Handle redrawing of OpenGL window
void COpenGLCtrl::RedrawOpenGLWindow() 
{
    ASSERT(m_hDC);
    ASSERT(!GetOpenGLError());
    ASSERT(wglGetCurrentContext());
    ASSERT(wglGetCurrentDC()==m_hDC);

    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT); 
    glPushMatrix();

    OnRender(); // Draw your scene
    ASSERT(!GetOpenGLError());

    glPopMatrix();
    glFinish();

    SwapBuffers(m_hDC);
    ASSERT(!GetOpenGLError());
}

//=============================================================================
// Here I implement a default 3D image so that something is drawn automatically
//=============================================================================

// One time initialization of rendering context
void COpenGLCtrl::OnCreateRC()
{
    glClearColor(0.0f,0.0f,0.0f,0.0f);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
}

// Render the scene
void COpenGLCtrl::OnRender()
{
    // Draw a color pyramid by default
    glBegin(GL_TRIANGLE_FAN);
       glColor3f(1.0f,1.0f,1.0f);
       glVertex3f(0.0f,1.0f,0.0f);
       glColor3f(1.0f,0.0f,0.0f);
       glVertex3f(-1.0f,0.0f,0.0f);
       glColor3f(1.0f,1.0f,0.0f);
       glVertex3f(0.0f,0.0f,1.0f);
       glColor3f(0.0f,1.0f,0.0f);
       glVertex3f(1.0f,0.0f,0.0f);
       glColor3f(0.0f,0.0f,1.0f);
       glVertex3f(0.0f,0.0f,-1.0f);
       glColor3f(1.0f,0.0f,0.0f);
       glVertex3f(-1.0f,0.0f,0.0f);
    glEnd();
}

// Setup viewport transformation
void COpenGLCtrl::OnViewport(int cx, int cy)
{
    glViewport(0,0,cx,cy); // Entire window
}

// Setup viewing transformation
void COpenGLCtrl::OnViewing()
{
    gluLookAt(2,2,2,0,0,0,0,1,0);
}

// Setup perspective transformation
void COpenGLCtrl::OnProjection(GLdouble AspectRatio)
{
    gluPerspective(45,AspectRatio,1,10);
}

// Any cleanup to do?
void COpenGLCtrl::OnDestroyRC()
{
}

//=============================================================================
// Class COpenGLCtrls to be used if you have more than one window with OpenGL
//=============================================================================

COpenGLCtrls::COpenGLCtrls()
             :m_CountBeginEnd(0)
{
}

COpenGLCtrls::~COpenGLCtrls()
{
    // You must call BeginOpenGLDrawing and EndOpenGLDrawing
    // in pairs, if this asserts, you did not match the calls
    ASSERT(m_CountBeginEnd==0);
}

void COpenGLCtrls::InitializeOpenGLWindow()
{
    COpenGLCtrl::InitializeOpenGLWindow();

    // Now make the RC not current
    ASSERT(m_hDC);
    VERIFY(wglMakeCurrent(NULL,NULL));
    VERIFY(::ReleaseDC(m_hWnd,m_hDC));
    m_hDC=NULL;
}


void COpenGLCtrls::UninitializeOpenGLWindow()
{
    // Make the RC current so that we can call OnDestroyRC in base class
    ASSERT(!m_hDC);
    VERIFY(m_hDC=::GetDC(m_hWnd));
    VERIFY(wglMakeCurrent(m_hDC,m_hRC));
    ASSERT(!GetOpenGLError());

    COpenGLCtrl::UninitializeOpenGLWindow();
}


void COpenGLCtrls::BeginOpenGLDrawing()
{
    ASSERT(m_CountBeginEnd>=0);
    ASSERT(!m_hDC==!m_CountBeginEnd); // Consistency
    
    if (m_CountBeginEnd++) 
        return;  // Already made current

    ASSERT(m_hRC); 
    ASSERT(!m_hDC);
    ASSERT(m_hWnd && ::IsWindow(m_hWnd)); 
    ASSERT(!wglGetCurrentContext());
    ASSERT(!wglGetCurrentDC());

    VERIFY(m_hDC=::GetDC(m_hWnd));
    VERIFY(wglMakeCurrent(m_hDC,m_hRC));
    ASSERT(!GetOpenGLError());
}

void COpenGLCtrls::EndOpenGLDrawing()
{
    ASSERT(m_CountBeginEnd>0);
    ASSERT(!m_hDC==!m_CountBeginEnd); // Consistency

    if (--m_CountBeginEnd) 
        return; // Keep current

    ASSERT(m_hRC); 
    ASSERT(m_hDC);
    ASSERT(m_hWnd && ::IsWindow(m_hWnd)); 
    ASSERT(wglGetCurrentContext());
    ASSERT(wglGetCurrentDC()==m_hDC);

    ASSERT(!GetOpenGLError());
    VERIFY(wglMakeCurrent(NULL,NULL));
    VERIFY(::ReleaseDC(m_hWnd,m_hDC));
    m_hDC=NULL; 
}

void COpenGLCtrls::ResizeOpenGLWindow(int cx, int cy) 
{
    BeginOpenGLDrawing();
    COpenGLCtrl::ResizeOpenGLWindow(cx,cy);
    EndOpenGLDrawing();
}

void COpenGLCtrls::RedrawOpenGLWindow() 
{
    BeginOpenGLDrawing();
    COpenGLCtrl::RedrawOpenGLWindow();
    EndOpenGLDrawing();
}

//=============================================================================
// Test for OpenGL errors
//=============================================================================

// See if there is an error and trace it
BOOL GetOpenGLError()
{
    GLenum Value=glGetError();
    LPCSTR Error;

    switch (Value)
    {
       case GL_NO_ERROR         : return FALSE;
       case GL_INVALID_ENUM     : Error="GL_INVALID_ENUM"     ; break;
       case GL_INVALID_VALUE    : Error="GL_INVALID_VALUE"    ; break;
       case GL_INVALID_OPERATION: Error="GL_INVALID_OPERATION"; break;
       case GL_STACK_OVERFLOW   : Error="GL_STACK_OVERFLOW"   ; break;
       case GL_STACK_UNDERFLOW  : Error="GL_STACK_UNDERFLOW"  ; break;
       case GL_OUT_OF_MEMORY    : Error="GL_OUT_OF_MEMORY"    ; break;
       default                  : Error="<Unknown>";ASSERT(0) ; break; 
    }

    TRACE1("OpenGL's glGetError() returned error %s\n",(LPCSTR)Error);
    return TRUE;
}

   