#ifndef OGLUTIL_H
#define OGLUTIL_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Include the standard OpenGL headers
#include "gl\gl.h"     
#include "gl\glu.h"
#include "gl\glaux.h"

#include "math.h"    // For sqrt
#include <stdexcept> // For runtime_error exception

// Static assert, derived from Andrei Alexandrescu's static_checker
// See Andrei's CUJ April'99 article on SAFEARRAYs
// Leave it also in release builds, it costs nothing
template <bool> struct static_checker;
template <> struct static_checker<true> {};
#define STATIC_ASSERT(expr) static_checker<expr>()

//******************************************************************************
// OpenGL reusable utility template classes GLpoint4 and GLvector3  
// By G.Bavestrelli, Copyright Techint S.p.A. 1998-1999
// Any feedback is welcome, you can contact me at giovanni-bavestrelli@pomini.it
//******************************************************************************

// This is a vector in three dimensional space
template <class Numeric>
class GLvector3
{

public:

  Numeric x;
  Numeric y;
  Numeric z;

  GLvector3()
      :x(0),y(0),z(0)
      {}

  GLvector3(Numeric x, Numeric y, Numeric z)
      :x(x),y(y),z(z)
	  {}

  // The default copy constructor works fine and 
  // is faster than a member by member assignment
  // The same is true for the assigment operator

  GLvector3(const Numeric * v)
      :x(v[0]),y(v[1]),z(v[2])
	  {}

  operator const Numeric * () const
  {
     // If this line does not compile, it's because the Numeric members are not aligned like C arrays
     STATIC_ASSERT(offsetof(GLvector3<Numeric>,y)-offsetof(GLvector3<Numeric>,x)==sizeof(Numeric));
     return &x; 
  }

  GLvector3 & operator =(const Numeric * v)
  {
      ASSERT(v);
      x=v[0]; y=v[1]; z=v[2]; 
      return *this; 
  }

  GLvector3 operator - ()
  {
      return GLvector3<Numeric>(-x,-y,-z); 
  }

  double GetLength() const 
  { 
      return sqrt(static_cast<double>(x*x+y*y+z*z)); 
  }

  bool Normalize() 
  {
     double Length=GetLength();
     if (Length==0.0) return false;
     x/=Length; y/=Length; z/=Length; 
     return true;
  }

  GLvector3 GetNormal() const 
  {
     double Length=GetLength();
     if (Length==0.0) return GLvector3();
     else return GLvector3(x/Length,y/Length,z/Length);
  }

};

template <class Numeric>
inline bool operator == (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{
    return V1.x==V2.x && V1.y==V2.y && V1.z==V2.z;
}

template <class Numeric>
inline bool operator != (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{
    return !(V1==V2);
}


// This is a homogeneous coordinate in three dimensional space
template <class Numeric>
class GLpoint4
{
 public:

   Numeric x;
   Numeric y;
   Numeric z;
   Numeric w;

   GLpoint4()
       :x(0),y(0),z(0),w(1)
       {}

   // The default copy constructor works fine and 
   // is faster than a member by member assignment
   // The same is true for the assigment operator

   GLpoint4(Numeric x,Numeric y,Numeric z=0,Numeric w=1)
       :x(x),y(y),z(z),w(w)
       {}

   GLpoint4(const Numeric * p)
       :x(p[0]),y(p[1]),z(p[2]),w(p[3])
       {}
 
   operator const Numeric * () const 
   { 
      // If this line does not compile, it's because the Numeric members are not aligned like C arrays
      STATIC_ASSERT(offsetof(GLpoint4<Numeric>,y)-offsetof(GLpoint4<Numeric>,x)==sizeof(Numeric));
      return &x; 
   }

   GLpoint4 & operator = (const Numeric * c)
   { 
      ASSERT(c);
      x=c[0]; y=c[1]; z=c[2]; w=c[3]; 
      return *this; 
   }

   GLpoint4 operator - ()
   {
      return GLpoint4<Numeric>(-x,-y,-z,w); 
   }

   bool Normalize()
   {
     if (w==0) return false;
     if (w==1) return true;
     x/=w; y/=w; z/=w; w=1; 
     return true;
   }
};

template <class Numeric>
inline bool operator == (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2)
{
    return C1.x==C2.x && C1.y==C2.y && C1.z==C2.z && C1.w==C2.w;
}

template <class Numeric>
inline bool operator != (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2)
{
    return !(C1==C2);
}



// Very useful template operators and functions that work on vectors and points

template <class Numeric>
inline GLvector3<Numeric> operator + (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{ 
  return GLvector3<Numeric>(V1.x+V2.x, V1.y+V2.y, V1.z+V2.z); 
}

template <class Numeric>
inline GLvector3<Numeric> operator - (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{ 
  return GLvector3<Numeric>(V1.x-V2.x, V1.y-V2.y, V1.z-V2.z); 
}

template <class Numeric>
inline GLpoint4<Numeric> operator + (const GLpoint4<Numeric> & P, const GLvector3<Numeric> & V) 
{ 
  return GLpoint4<Numeric>(P.x+V.x, P.y+V.y, P.z+V.z); 
}

template <class Numeric> // Cross product
inline GLvector3<Numeric> operator * (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2) 
{ 
  return GLvector3<Numeric>(V1.y*V2.z-V2.y*V1.z, V2.x*V1.z-V1.x*V2.z, V1.x*V2.y-V2.x*V1.y); 
}

template <class Numeric> // Explicit Cross Product
inline GLvector3<Numeric> CrossProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{
   return GLvector3<Numeric>(V1.y*V2.z-V2.y*V1.z, V2.x*V1.z-V1.x*V2.z, V1.x*V2.y-V2.x*V1.y); 
}

template <class Numeric>
inline Numeric DotProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{ 
    return V1.x*V2.x+V1.y*V2.y+V1.z*V2.z; 
}

template <class Numeric>
GLvector3<Numeric> NormalizedCrossProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{
      GLvector3<Numeric> V(V1.y*V2.z-V2.y*V1.z, V2.x*V1.z-V1.x*V2.z, V1.x*V2.y-V2.x*V1.y);  
      V.Normalize();
      return V;
}

template <class Numeric>
GLvector3<Numeric> operator - (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2) throw(std::runtime_error)
{ 
      ASSERT(C1.w!=0 && C2.w!=0); // Check that points are not at infinity
      if (C1.w==0 || C2.w==0) 
          throw std::runtime_error("Cannot subtract points if any of them is at infinity");
      else if (C1.w==1 && C2.w==1) 
          return GLvector3<Numeric>(C1.x-C2.x, C1.y-C2.y, C1.z-C2.z);
      else
          return GLvector3<Numeric>(C1.x/C1.w-C2.x/C2.w, C1.y/C1.w-C2.y/C2.w, C1.z/C1.w-C2.z/C2.w);
}

#endif